@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Linguist of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 /*  TRANSLATOR MessageEditor
 
@@ -38,6 +13,7 @@
 
 #include <QApplication>
 #include <QBoxLayout>
+#include <QCheckBox>
 #ifndef QT_NO_CLIPBOARD
 #include <QClipboard>
 #endif
@@ -52,6 +28,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::Literals::StringLiterals;
+
 /*
    MessageEditor class impl.
 
@@ -72,7 +50,7 @@ MessageEditor::MessageEditor(MultiDataModel *dataModel, QMainWindow *parent)
       m_selectionHolder(0),
       m_focusWidget(0)
 {
-    setObjectName(QLatin1String("scroll area"));
+    setObjectName("scroll area"_L1);
 
     QPalette p;
     p.setBrush(QPalette::Window, p.brush(QPalette::Active, QPalette::Base));
@@ -119,7 +97,7 @@ void MessageEditor::setupEditorPage()
     editorPage->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
 
     m_source = new FormWidget(tr("Source text"), false);
-    m_source->setHideWhenEmpty(true);
+    m_source->setHideWhenEmpty(false);
     m_source->setWhatsThis(tr("This area shows the source text."));
     connect(m_source, &FormWidget::selectionChanged,
             this, &MessageEditor::selectionChanged);
@@ -132,13 +110,19 @@ void MessageEditor::setupEditorPage()
 
     m_commentText = new FormWidget(tr("Developer comments"), false);
     m_commentText->setHideWhenEmpty(true);
-    m_commentText->setObjectName(QLatin1String("comment/context view"));
+    m_commentText->setObjectName("comment/context view"_L1);
     m_commentText->setWhatsThis(tr("This area shows a comment that"
                         " may guide you, and the context in which the text"
                         " occurs.") );
     connect(m_commentText, &FormWidget::selectionChanged,
             this, &MessageEditor::selectionChanged);
 
+    m_ncrModeBox = new QCheckBox(tr("NCR mode"));
+    m_ncrModeBox->setWhatsThis(tr("Toggles Numeric Character Reference Mode "
+                                  "for displaying the source text and the translations."));
+    m_ncrModeBox->setHidden(true);
+    connect(m_ncrModeBox, &QCheckBox::checkStateChanged, this, &MessageEditor::toggleNcrMode);
+
     QBoxLayout *subLayout = new QVBoxLayout;
 
     subLayout->setContentsMargins(5, 5, 5, 5);
@@ -146,10 +130,14 @@ void MessageEditor::setupEditorPage()
     subLayout->addWidget(m_pluralSource);
     subLayout->addWidget(m_commentText);
 
+    QBoxLayout *horizontalLayout = new QHBoxLayout;
+    horizontalLayout->addLayout(subLayout);
+    horizontalLayout->addWidget(m_ncrModeBox, 0, Qt::AlignTop | Qt::AlignRight);
+
     m_layout = new QVBoxLayout;
     m_layout->setSpacing(2);
     m_layout->setContentsMargins(2, 2, 2, 2);
-    m_layout->addLayout(subLayout);
+    m_layout->addLayout(horizontalLayout);
     m_layout->addStretch(1);
     editorPage->setLayout(m_layout);
 
@@ -217,7 +205,7 @@ void MessageEditor::messageModelAppended()
 
 void MessageEditor::allModelsDeleted()
 {
-    for (const MessageEditorData &med : qAsConst(m_editors))
+    for (const MessageEditorData &med : std::as_const(m_editors))
         med.container->deleteLater();
     m_editors.clear();
     m_currentModel = -1;
@@ -261,7 +249,7 @@ void MessageEditor::addPluralForm(int model, const QString &label, bool writable
         transEditor->setVisible(false);
     transEditor->setMultiEnabled(m_lengthVariants);
     static_cast<QBoxLayout *>(m_editors[model].container->layout())->insertWidget(
-        m_editors[model].transTexts.count(), transEditor);
+        m_editors[model].transTexts.size(), transEditor);
 
     connect(transEditor, &FormMultiWidget::selectionChanged,
             this, &MessageEditor::selectionChanged);
@@ -318,7 +306,7 @@ void MessageEditor::fixTabOrder()
 void MessageEditor::reallyFixTabOrder()
 {
     QWidget *prev = this;
-    for (const MessageEditorData &med : qAsConst(m_editors)) {
+    for (const MessageEditorData &med : std::as_const(m_editors)) {
         for (FormMultiWidget *fmw : med.transTexts)
             for (QTextEdit *te : fmw->getEditors()) {
                 setTabOrder(prev, te);
@@ -340,7 +328,7 @@ void MessageEditor::reallyFixTabOrder()
 QStringList MessageEditor::translations(int model) const
 {
     QStringList translations;
-    for (int i = 0; i < m_editors[model].transTexts.count() &&
+    for (int i = 0; i < m_editors[model].transTexts.size() &&
                     m_editors[model].transTexts.at(i)->isVisible(); ++i)
         translations << m_editors[model].transTexts[i]->getTranslation();
     return translations;
@@ -392,7 +380,7 @@ void MessageEditor::resetSelection()
             disconnect(fte, &FormatTextEdit::editorDestroyed,
                        this, &MessageEditor::editorDestroyed);
         }
-        m_selectionHolder = 0;
+        m_selectionHolder = nullptr;
 #ifndef QT_NO_CLIPBOARD
         updateCanCutCopy();
 #endif
@@ -401,8 +389,8 @@ void MessageEditor::resetSelection()
 
 void MessageEditor::activeModelAndNumerus(int *model, int *numerus) const
 {
-    for (int j = 0; j < m_editors.count(); ++j) {
-        for (int i = 0; i < m_editors[j].transTexts.count(); ++i)
+    for (int j = 0; j < m_editors.size(); ++j) {
+        for (int i = 0; i < m_editors[j].transTexts.size(); ++i)
             for (QTextEdit *te : m_editors[j].transTexts[i]->getEditors())
                 if (m_focusWidget == te) {
                     *model = j;
@@ -468,11 +456,11 @@ void MessageEditor::setTargetLanguage(int model)
 {
     const QStringList &numerusForms = m_dataModel->model(model)->numerusForms();
     const QString &langLocalized = m_dataModel->model(model)->localizedLanguage();
-    for (int i = 0; i < numerusForms.count(); ++i) {
+    for (int i = 0; i < numerusForms.size(); ++i) {
         const QString &label = tr("Translation to %1 (%2)").arg(langLocalized, numerusForms[i]);
         if (!i)
             m_editors[model].firstForm = label;
-        if (i >= m_editors[model].transTexts.count())
+        if (i >= m_editors[model].transTexts.size())
             addPluralForm(model, label, m_dataModel->isModelWritable(model));
         else
             m_editors[model].transTexts[i]->setLabel(label);
@@ -481,16 +469,18 @@ void MessageEditor::setTargetLanguage(int model)
                 tr("This is where you can enter or modify"
                    " the translation of the above source text.") );
     }
-    for (int j = m_editors[model].transTexts.count() - numerusForms.count(); j > 0; --j)
+    for (int j = m_editors[model].transTexts.size() - numerusForms.size(); j > 0; --j)
         delete m_editors[model].transTexts.takeLast();
     m_editors[model].invariantForm = tr("Translation to %1").arg(langLocalized);
     m_editors[model].transCommentText->setLabel(tr("Translator comments for %1").arg(langLocalized));
+    m_editors[model].container->setToolTip(
+            QLocale::languageToString(m_dataModel->model(model)->language()));
 }
 
 MessageEditorData *MessageEditor::modelForWidget(const QObject *o)
 {
-    for (int j = 0; j < m_editors.count(); ++j) {
-        for (int i = 0; i < m_editors[j].transTexts.count(); ++i)
+    for (int j = 0; j < m_editors.size(); ++j) {
+        for (int i = 0; i < m_editors[j].transTexts.size(); ++i)
             for (QTextEdit *te : m_editors[j].transTexts[i]->getEditors())
                 if (te == o)
                     return &m_editors[j];
@@ -535,6 +525,17 @@ bool MessageEditor::eventFilter(QObject *o, QEvent *e)
         QWidget *widget = static_cast<QWidget *>(o);
         if (widget != m_focusWidget)
             trackFocus(widget);
+    } else if (e->type() == QEvent::ApplicationPaletteChange
+               || e->type() == QEvent::PaletteChange) {
+        QPalette p;
+        p.setBrush(QPalette::Window, p.brush(QPalette::Active, QPalette::Base));
+        setPalette(p);
+        if (m_editors.size() > 1) {
+            for (qsizetype i = 0; i < m_editors.size(); i++) {
+                m_editors[i].container->setPalette(paletteForModel(i));
+                m_editors[i].container->setAutoFillBackground(true);
+            }
+        }
     }
 
     return QScrollArea::eventFilter(o, e);
@@ -559,7 +560,6 @@ void MessageEditor::trackFocus(QWidget *widget)
         m_currentModel = model;
         m_currentNumerus = numerus;
         emit activeModelChanged(activeModel());
-        updateBeginFromSource();
         updateUndoRedo();
 #ifndef QT_NO_CLIPBOARD
         updateCanPaste();
@@ -572,19 +572,41 @@ void MessageEditor::showNothing()
     m_source->clearTranslation();
     m_pluralSource->clearTranslation();
     m_commentText->clearTranslation();
-    for (int j = 0; j < m_editors.count(); ++j) {
+    m_ncrModeBox->setHidden(true);
+
+    for (int j = 0; j < m_editors.size(); ++j) {
         setEditingEnabled(j, false);
-        for (FormMultiWidget *widget : qAsConst(m_editors[j].transTexts))
+        for (FormMultiWidget *widget : std::as_const(m_editors[j].transTexts))
             widget->clearTranslation();
         m_editors[j].transCommentText->clearTranslation();
     }
 #ifndef QT_NO_CLIPBOARD
     emit pasteAvailable(false);
 #endif
-    updateBeginFromSource();
     updateUndoRedo();
 }
 
+void MessageEditor::toggleNcrMode()
+{
+    for (int j = 0; j < m_editors.size(); ++j) {
+
+        MessageItem *currentMessage = m_dataModel->messageItem(m_currentIndex, j);
+        if (!currentMessage)
+            continue;
+
+        bool newNcrMode = m_ncrModeBox->isChecked();
+        if (currentMessage->ncrMode() != newNcrMode) {
+            currentMessage->setNcrMode(newNcrMode);
+            m_source->setTranslation(currentMessage->text());
+            m_pluralSource->setTranslation(currentMessage->pluralText());
+            auto translations = currentMessage->translations();
+            for (int i = 0; i < translations.size(); i++)
+                setNumerusTranslation(j, translations.at(i), i);
+        }
+        break;
+    }
+}
+
 void MessageEditor::showMessage(const MultiDataIndex &index)
 {
     m_currentIndex = index;
@@ -611,7 +633,7 @@ void MessageEditor::showMessage(const MultiDataIndex &index)
                 QString toolTip = tr("'%1'\nLine: %2").arg(item->fileName(), QString::number(item->lineNumber()));
                 m_source->setToolTip(toolTip);
             } else {
-                m_source->setToolTip(QLatin1String(""));
+                m_source->setToolTip({});
             }
 
             // Comment field
@@ -619,7 +641,7 @@ void MessageEditor::showMessage(const MultiDataIndex &index)
 
             if (!item->extraComment().isEmpty()) {
                 if (!commentText.isEmpty())
-                    commentText += QLatin1String("\n");
+                    commentText += u'\n';
                 commentText += item->extraComment().simplified();
             }
 
@@ -633,27 +655,25 @@ void MessageEditor::showMessage(const MultiDataIndex &index)
                              && item->message().type() != TranslatorMessage::Vanished);
 
         // Translation label
-        ed.pluralEditMode = item->translations().count() > 1;
+        ed.pluralEditMode = item->translations().size() > 1;
         ed.transTexts.first()->setLabel(ed.pluralEditMode ? ed.firstForm : ed.invariantForm);
 
         // Translation forms
-        if (item->text().isEmpty() && !item->context().isEmpty()) {
-            for (int i = 0; i < ed.transTexts.size(); ++i)
-                ed.transTexts.at(i)->setVisible(false);
-        } else {
-            QStringList normalizedTranslations =
-                m_dataModel->model(j)->normalizedTranslations(*item);
-            for (int i = 0; i < ed.transTexts.size(); ++i) {
-                bool shouldShow = (i < normalizedTranslations.count());
-                if (shouldShow)
-                    setNumerusTranslation(j, normalizedTranslations.at(i), i);
-                else
-                    setNumerusTranslation(j, QString(), i);
-                ed.transTexts.at(i)->setVisible(i == 0 || shouldShow);
-            }
+        QStringList normalizedTranslations =
+            m_dataModel->model(j)->normalizedTranslations(*item);
+        for (int i = 0; i < ed.transTexts.size(); ++i) {
+            bool shouldShow = (i < normalizedTranslations.size());
+            if (shouldShow)
+                setNumerusTranslation(j, normalizedTranslations.at(i), i);
+            else
+                setNumerusTranslation(j, QString(), i);
+            ed.transTexts.at(i)->setVisible(i == 0 || shouldShow);
         }
 
         ed.transCommentText->setTranslation(item->translatorComment().trimmed(), false);
+
+        m_ncrModeBox->setCheckState(item->ncrMode() ? Qt::Checked : Qt::Unchecked);
+        m_ncrModeBox->setHidden(false);
     }
 
     updateUndoRedo();
@@ -662,12 +682,10 @@ void MessageEditor::showMessage(const MultiDataIndex &index)
 void MessageEditor::setNumerusTranslation(int model, const QString &translation, int numerus)
 {
     MessageEditorData &ed = m_editors[model];
-    if (numerus >= ed.transTexts.count())
+    if (numerus >= ed.transTexts.size())
         numerus = 0;
     FormMultiWidget *transForm = ed.transTexts[numerus];
     transForm->setTranslation(translation, false);
-
-    updateBeginFromSource();
 }
 
 void MessageEditor::setTranslation(int latestModel, const QString &translation)
@@ -682,14 +700,12 @@ void MessageEditor::setTranslation(int latestModel, const QString &translation)
     FormMultiWidget *transForm = m_editors[latestModel].transTexts[numerus];
     transForm->getEditors().first()->setFocus();
     transForm->setTranslation(translation, true);
-
-    updateBeginFromSource();
 }
 
 void MessageEditor::setEditingEnabled(int model, bool enabled)
 {
     MessageEditorData &ed = m_editors[model];
-    for (FormMultiWidget *widget : qAsConst(ed.transTexts))
+    for (FormMultiWidget *widget : std::as_const(ed.transTexts))
         widget->setEditingEnabled(enabled);
     ed.transCommentText->setEditingEnabled(enabled);
 
@@ -701,7 +717,7 @@ void MessageEditor::setEditingEnabled(int model, bool enabled)
 void MessageEditor::setLengthVariants(bool on)
 {
     m_lengthVariants = on;
-    for (const MessageEditorData &ed : qAsConst(m_editors))
+    for (const MessageEditorData &ed : std::as_const(m_editors))
         for (FormMultiWidget *widget : ed.transTexts)
             widget->setMultiEnabled(on);
 }
@@ -791,19 +807,16 @@ void MessageEditor::clipboardChanged()
 
 void MessageEditor::selectAll()
 {
-    // make sure we don't select the selection of a translator textedit,
-    // if we really want the source text editor to be selected.
     QTextEdit *te;
-    if ((te = m_source->getEditor())->underMouse()
-        || (te = m_pluralSource->getEditor())->underMouse()
-        || ((te = activeEditor()) && te->hasFocus()))
+    if (((te = activeEditor()) && te->hasFocus())
+        || (te = m_source->getEditor())->underMouse()
+        || (te = m_pluralSource->getEditor())->underMouse())
         te->selectAll();
 }
 
 void MessageEditor::emitTranslationChanged(QTextEdit *widget)
 {
     grabFocus(widget); // DND proofness
-    updateBeginFromSource();
     updateUndoRedo();
     emit translationChanged(translations(m_currentModel));
 }
@@ -815,17 +828,10 @@ void MessageEditor::emitTranslatorCommentChanged(QTextEdit *widget)
     emit translatorCommentChanged(m_editors[m_currentModel].transCommentText->getTranslation());
 }
 
-void MessageEditor::updateBeginFromSource()
-{
-    bool overwrite = false;
-    if (QTextEdit *activeEditor = activeTranslation())
-        overwrite = !activeEditor->isReadOnly()
-            && activeEditor->toPlainText().trimmed().isEmpty();
-    emit beginFromSourceAvailable(overwrite);
-}
-
 void MessageEditor::beginFromSource()
 {
+    if (m_currentModel < 0 || m_currentIndex.model() < 0)
+        return;
     MessageItem *item = m_dataModel->messageItem(m_currentIndex, m_currentModel);
     setTranslation(m_currentModel,
                    m_currentNumerus > 0 && !item->pluralText().isEmpty() ?
@@ -848,7 +854,6 @@ void MessageEditor::setEditorFocusForModel(int model)
             m_currentModel = -1;
             m_focusWidget = 0;
             emit activeModelChanged(activeModel());
-            updateBeginFromSource();
             updateUndoRedo();
 #ifndef QT_NO_CLIPBOARD
             updateCanPaste();
@@ -861,7 +866,7 @@ void MessageEditor::setEditorFocusForModel(int model)
 
 bool MessageEditor::focusNextUnfinished(int start)
 {
-    for (int j = start; j < m_editors.count(); ++j)
+    for (int j = start; j < m_editors.size(); ++j)
         if (m_dataModel->isModelWritable(j))
             if (MessageItem *item = m_dataModel->messageItem(m_currentIndex, j))
                 if (item->type() == TranslatorMessage::Unfinished) {
@@ -888,7 +893,7 @@ void MessageEditor::setVisualizeWhitespace(bool value)
     m_pluralSource->getEditor()->setVisualizeWhitespace(value);
     m_commentText->getEditor()->setVisualizeWhitespace(value);
 
-    for (const MessageEditorData &med : qAsConst(m_editors)) {
+    for (const MessageEditorData &med : std::as_const(m_editors)) {
         med.transCommentText->getEditor()->setVisualizeWhitespace(value);
         for (FormMultiWidget *widget : med.transTexts)
             for (FormatTextEdit *te : widget->getEditors())
@@ -918,7 +923,7 @@ void MessageEditor::applyFontSize()
     m_pluralSource->getEditor()->setFont(font);
     m_commentText->getEditor()->setFont(font);
 
-    for (const MessageEditorData &med : qAsConst(m_editors)) {
+    for (const MessageEditorData &med : std::as_const(m_editors)) {
         for (FormMultiWidget *fmw : med.transTexts)
             for (QTextEdit *te : fmw->getEditors())
                 te->setFont(font);
